@@ -26,6 +26,7 @@
 #include "types.h"
 #include "rip_manager.h"
 #include "util.h"
+#define SRVERSION	"1.32"
 
 /*******************************************************************************
  * Private functions
@@ -162,7 +163,7 @@ void print_status()
 			   m_curinfo.server_name,
 			   m_curinfo.bitrate,
 			   m_curinfo.meta_interval);
-		if(!OPT_FLAG_ISSET(m_opt.flags, OPT_NO_RELAY))
+		if(GET_MAKE_RELAY(m_opt.flags))
 		{
 			fprintf(stderr, "relay port: %d\n"
 					"[%14s]\r",
@@ -226,6 +227,7 @@ void print_usage()
         fprintf(stderr, "        -l <seconds>   - number of seconds to run, otherwise runs forever\n");
         fprintf(stderr, "        -q             - add sequence number to output file\n");
         fprintf(stderr, "        -i             - dont add ID3V1 Tags to output file\n");
+        fprintf(stderr, "        -u <useragent> - Use a different UserAgent then \"Streamripper\"\n");
 
 }
 
@@ -252,12 +254,12 @@ void parse_arguments(int argc, char **argv)
 	m_opt.flags = OPT_AUTO_RECONNECT | 
 		      OPT_SEPERATE_DIRS | 
 		      OPT_SEARCH_PORTS |
-		      OPT_NO_RELAY |
 			  OPT_ADD_ID3;
 
 	strcpy(m_opt.output_directory, "./");
 	m_opt.proxyurl[0] = (char)NULL;
 	strncpy(m_opt.url, argv[1], MAX_URL_LEN);
+	strcpy(m_opt.useragent, "sr-POSIX/1.32");
 
         //get arguments
         for(i = 1; i < argc; i++)
@@ -265,7 +267,7 @@ void parse_arguments(int argc, char **argv)
                 if (argv[i][0] != '-')
                         continue;
 
-                c = strchr("dpl", argv[i][1]);
+                c = strchr("dplu", argv[i][1]);
                 if (c != NULL)
                         if ((i == (argc-1)) || (argv[i+1][0] == '-'))
                         {
@@ -289,7 +291,7 @@ void parse_arguments(int argc, char **argv)
                                 m_opt.flags ^= OPT_SEPERATE_DIRS;
                                 break;
                         case 'r':
-                                m_opt.flags ^= OPT_NO_RELAY;
+                                m_opt.flags ^= OPT_MAKE_RELAY;
 								// Default
 								if (i == (argc-1) || argv[i+1][0] == '-')
 									break;
@@ -312,13 +314,17 @@ void parse_arguments(int argc, char **argv)
                                 m_opt.flags ^= OPT_AUTO_RECONNECT;
                                 break;
                         case 'v':
-								printf("streamripper 1.0.5 by Jon Clegg <jonclegg@yahoo.com>\n");
-								exit(0);
+				printf("Streamripper %s by Jon Clegg <jonclegg@yahoo.com>\n", SRVERSION);
+				exit(0);
                         case 'l':
-								i++;
-								time(&m_stop_time);
-								m_stop_time += atoi(argv[i]);
-								break;
+				i++;
+				time(&m_stop_time);
+				m_stop_time += atoi(argv[i]);
+				break;
+                        case 'u':
+				i++;
+				strncpy(m_opt.useragent, argv[i], MAX_USERAGENT_STR);
+				break;
                 }
         }
 }
